/*global jQuery, ManageLayoutsDWR */

// Layout Data Loading Agent

// binds: atb-layout-select, atb-layout-reload, atb-layout-refresh
// triggers: atb-layout-updated, atb-layout-cancel

// requires: jquery.datatypes.js, jquery.overlay.js, jquery.defer.js

/* To select/load a layout trigger an event on an element inside a component that
 * is to be updated:
 *
 * $(...).trigger('atb-layout-select', [ layoutId, options ])
 *
 * If triggered on or under an element with class 'atb-layout-data', then that
 * element is used as the source of options and will be where the layout data
 * and cache is stored.
 *
 * Options read from attributes of the .atb-layout-data element:
 *   data-layout-cache (boolean) - whether the layouts are to be cached
 *   data-layout-perms (boolean) - whether permission info is required
 *   data-layout-edits (boolean) - whether to create editing layers
 *
 * Additional options can be supplied as an object of boolean flags:
 *   reload - reload the layout (ignore any cached copy)
 *   keepEdits - keep any existing modifications in the edits and dialog layers 
 *   keepDialog - keep any existing mods in the dialog layer
 *
 * The layout will be loaded (or fetched from the cache), layered and stored in
 * the .atb-layout-data element. An 'atb-layout-updated' event will be triggered
 * on the original element that 'atb-layout-select' was triggered from.
 *
 * The layered data (in atb-layout-layers) takes the form:
 * {
 *     parent: ... the flattern parent layout
 *     overlay: ... the data of the current layout overlaying 'parent'
 *     edits: ... an object overlaying 'overlay', to hold modifications
 *     dialog: ... overlaying 'edits' for use by dialogs
 * }
 *
 * See jquery.overlay.js for more details about overlaying objects.
 */
(function($) {

	function dataContext(event) {
		return $(event.target).closest('.atb-layout-data');
	}

	$(document)
	.bind('atb-layout-select.layout-loader', function(event, layoutId, opts) {
		var context = dataContext(event),
			useCache = $.dt.bool(context.attr('data-layout-cache')),
			addPerms = $.dt.bool(context.attr('data-layout-perms')),
			addEdits = $.dt.bool(context.attr('data-layout-edits')),
			keepEdits = opts && opts.keepEdits,
			keepDialog = opts && (opts.keepEdits || opts.keepDialog),
			reload = opts && opts.reload,
			layers = context.data('atb-layout-layers') || {},
			cacheName = 'atb-cache-' + layoutId,
			// Fetch layout data from cache
			data = useCache && !reload && context.data(cacheName);

		// If the requested layout is already current, and a reload has not been forced
		if ( !reload && layers.overlay && layers.overlay.layoutId === layoutId ) {
			// Trigger a cancel event
			$(event.target).trigger('atb-layout-cancel');
			return;
		}

		function loaded(data) {
			var layoutData;

			layers.parent = data.parent;
			layers.overlay = data.parent ? $.overlay(data.parent, data.overlay, {notNulls:true}) : data.overlay;

			function addEditLayer(underlay, overlay, keep) {
				return keep && overlay ?
					// Keep existing edits
					$.overlay(underlay, overlay, {notNulls:true,notEquals:true,onlyOwn:true}) :
					// Create a fresh edits overlay
					$.overlay(underlay);
			}

			if ( addEdits ) {
				// Keep existing edits
				layers.edits = addEditLayer(layers.overlay, layers.edits, keepEdits);
				layers.dialog = addEditLayer(layers.edits, layers.dialog, keepDialog);
			} else {
				delete layers.dialog;
				delete layers.edits;
			}

			layoutData = layers.edits || layers.overlay;

			if ( addPerms ) {
				// TODO: ManageLayoutsDWR should return this structure as-is:
				(function() {
					var globalPerms = context.data('atb-permissions') || {},
						perms = {
							view: data.viewPermission,
							edit: data.editPermission,
							panel: data.editPanelPermission,
							child: data.childPermission,
							manage: !!globalPerms.manageLayouts,
							css: !!globalPerms.editCSS,
							advanced: !!globalPerms.editAdvanced
						};

					layers.overlay.hasPermission = function( name ) {
						return !!perms[name];
					};
				})();
			}
			
			// Add extra data to each panel data object - we use a function so
			// that it is ignored when the layout data is saved via a DWR call.
			$.each(layers.parent || layers.overlay, function(key, obj) {
				// Is obj a panel data object?
				if ( obj !== null && typeof obj === 'object' && 'backgroundAttachment' in obj ) {

					// Get the panel id
					obj.panelId = function() {
						return key;
					};
					
					// Get the panel name
					obj.panelName = function() {
						return $('#layout-manager-preview [data-panel='+key+'][title]').attr('title');
					};
				}
			});
			
			// Add layer names
			$.each(layers, function(key, data) {
				if ( data ) {
					data.getLayerId = function() {
						return key;
					};
				}
			});

			context
				.data('atb-layout-layers', layers)
				.data('atb-layout', layoutData);

			$(event.target)
				.trigger('atb-layout-updated', [ layoutId, layoutData, layers ]);		
		}

		if ( data ) {
			// Data already present in cache
			loaded(data);
		} else {
			// Load layout data
			ManageLayoutsDWR.getLayout(layoutId, $.defer(0, function(data) {

				// Fix permissions of DEFAULT layout
				if ( layoutId === 'DEFAULT' ) {
					data.editPermission = false;
					data.editPanelPermission = false;
				}

				function fixLayout( data ) {
					// Ensure that a resources object is present
					if ( data && !data.resources ) {
						data.resources = {};
					}
				}

				fixLayout(data.parent);
				fixLayout(data.overlay);

				if ( useCache ) {
					// Cache the layout
					context.data(cacheName, data);
				}

				loaded(data);
			}));
		}
	})

	// Reload the current layout
	.bind('atb-layout-reload.layout-loader', function(event, opts) {
		$(event.target).trigger('atb-layout-select', [ dataContext(event).data('atb-layout').layoutId, $.extend({ noConfirm:true, reload:true }, opts) ]);
	})

	// Refresh the current layout
	.bind('atb-layout-refresh.layout-loader', function(event) {
		var context = dataContext(event),
			layoutData = context.data('atb-layout'),
			layers = context.data('atb-layout-layers');
		
		if ( layoutData ) {
			$(event.target).trigger('atb-layout-updated', [ layoutData.layoutId, layoutData, layers ]);
		}
	});

})(jQuery);

