/*global jQuery, window, confirm */

// Dialogs Agent

// This agent causes dialog fields to be committed to the 'dialog' layer,
// and intercepts atb-field-changed events preventing propagation.
// When user clicks 'Apply' or 'Ok', the 'dialog' layer values are copied to
// the 'edits'. If 'Cancel' is clicked the 'dialog' layer is cleared.

(function($) {

	$('#layout-manager :role(dialog)')
		.live('atb-init', function() {			

			var dialog = $(this),
				tabs = $(':role(tab)', dialog),
				tabpanels = $(':role(tabpanel)', dialog),
				context = $(this).closest('.atb-layout-data'),
				isPanelEditor = dialog.is('#layout-manager-panel-editor');

			dialog
				.addClass('atb-has-dialog-layer')
				.attr('data-layer', 'dialog');
			
			// Wrap tab text with a <label> so they get styled according to their data-origin
			$(tabs)
				.attr('data-origin', 'unknown')
				.find('a:not(:has(label))').wrapInner('<label/>').end();

			// Set an aggregated origin on each tab of the dialog
			function updateTabOrigin() {
				var tabpanel = this;
				$.each(['dialog', 'edits', 'overlay', 'parent'], function(i, origin) {
					if ( $('[data-origin~='+origin+']', tabpanel).length ) {
						$(tabpanel).attr('data-agg-origin', origin);
						$.roles.masters(tabpanel).attr('data-origin', origin);
						return false;
					}					
				});
			}
			
			function updateAllTabs() {
				window.setTimeout(function() {
					tabpanels.each(updateTabOrigin);
				}, 0);			
			}

			// We can be sure modifications exist if a value originates from 'dialog' but not 'edits'
			function updateModified() {
				var modified = !!$('[data-origin~=dialog]', dialog).length;
				dialog.toggleClass('dialog-modified', modified);
				
				// Disable the Apply/Ok/Cancel buttons if not modified
				$('a.dialog-apply,a.dialog-cancel', dialog).attr('aria-disabled', !modified);
				// NOTE: Close/Cancel buttons will switch via a css rule
			}
			
			function init() {
				tabs.attr('data-origin', 'unknown');
				updateModified();
				updateAllTabs();
			}

			function applyFields(commit, close) {
				var layers = context.data('atb-layout-layers'),
					modified = dialog.hasClass('dialog-modified');

				if ( modified ) {
					$('.layout-field', dialog).each(function() {
						var field = $(this).atbField(),
							pair = field.getDataPair(layers.dialog),
							value = pair.obj[pair.prop];

						if ( commit ) {
							// Copy value from dialog layer into edits layer
							field.setDataValue(value, layers.edits);
						}

						// Clear the value in the dialog layer
						delete pair.obj[pair.prop];

						// Recalc origins
						field.setViewOrigin(commit ? value : field.getDataValue());
					});

					if ( commit ) {
						if ( isPanelEditor ) {
							// Force layout preview to be updated
							var panelId = dialog.atbField().getDataValue().panelId();
							dialog.trigger('atb-panel-changed', [ panelId ]);	
						}
						context.trigger('atb-field-changed');
					}

					// Clear the modified status and update tabs
					init();
				}

				if ( close ) {
					dialog.attr('aria-hidden', true);
				}
			}

			context
				.bind('atb-layout-updated', init);

			dialog
				.bind('atb-ready', init)

				.bind('atb-field-changed', function(event, field) {
					// Update the modified state of the dialog
					updateModified();
					
					// Special case for panel editor dialog - update preview
					if ( isPanelEditor && field ) {
						var pair = field.getDataPair();
						// Is this a panel field?
						if ( pair.obj.panelId ) {
							$('#panel-display').atbPanelCSS(pair.prop, pair.obj[pair.prop], $('#layout-manager').data('atb-palette') || {});
						}
					}

					// Prevent propagation beyond the dialog
					event.stopPropagation();
				})

				.bind('dialog-apply', function(event) {
					applyFields(true, false);
				})

				.bind('dialog-cancel', function(event) {
					applyFields(false, true);
				})

				.bind('dialog-ok', function(event) {
					applyFields(true, true);
				})

				.bind('dialog-undo', function(event) {
					applyFields(false, false);
				});
			
			// Update data origin indicator on tabs
			tabpanels
				.bind('atb-field-changed', updateTabOrigin);
		});

	// Ask user if dialog edits should be committed before saving
	$('#layout-manager')
		.bind('atb-layout-save', function(event) {
			var modifiedDialogs = $('.dialog-modified', this);
			if ( modifiedDialogs.length ) {
				// TODO: fetch message from markup, i18n
				if ( confirm('There are unapplied changes in dialogs.\nApply these changes before saving?') ) {
					modifiedDialogs.trigger('dialog-apply');
				}
			}
		});

})(jQuery);

